<?php

namespace Drupal\decoupled_kit_menu\Controller;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class Default Controller.
 */
class DefaultController extends ControllerBase {

  /**
   * Drupal\Core\Menu\MenuLinkTreeInterface definition.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menuLinkTree;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->menuLinkTree = $container->get('menu.link_tree');
    return $instance;
  }

  /**
   * Get menu data.
   *
   * @param string $id
   *   Menu id.
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse
   *   Return menu data.
   */
  public function index($id) {
    $parameters = $this->menuLinkTree->getCurrentRouteMenuTreeParameters($id);
    $tree = $this->menuLinkTree->load($id, $parameters);
    $menu = [];

    if ($tree) {
      $manipulators = [
      ['callable' => 'menu.default_tree_manipulators:checkNodeAccess'],
      ['callable' => 'menu.default_tree_manipulators:checkAccess'],
      ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
      ];
      $tree = $this->menuLinkTree->transform($tree, $manipulators);
      $menu = $this->menuLinkTree->build($tree);

      // Get menu item url for all levels of the menu.
      if (!empty($menu['#items'])) {
        $menu['#items'] = $this->buildMenu($menu['#items']);
      }
    }

    // Save to cache.
    if (!empty($menu)) {
      // Cache context.
      if (isset($menu['#cache']['contexts'])) {
        $menu['#cache']['contexts'] = Cache::mergeTags($menu['#cache']['contexts'], ['languages']);
      }

      // Cache tag for menu changes.
      if (isset($menu['#cache']['tags'])) {
        $menu['#cache']['tags'] = Cache::mergeTags($menu['#cache']['tags'], ['config:system.menu.' . $id]);
      }
    }

    // Output.
    $array = [
      'id' => $id,
      'data' => $menu,
    ];

    $response = new CacheableJsonResponse($array);

    if (!empty($menu['#cache']['contexts'])) {
      $cacheMeta = (new CacheableMetadata())->addCacheContexts($menu['#cache']['contexts']);
      $response->addCacheableDependency($cacheMeta);
    }
    if (!empty($menu['#cache']['tags'])) {
      $cacheMeta = (new CacheableMetadata())->addCacheTags($menu['#cache']['tags']);
      $response->addCacheableDependency($cacheMeta);
    }

    return $response;
  }

  /**
   * Recursive function to get menu items url from all levels.
   *
   * @param array $menu_items
   *   Array of menu items.
   *
   * @return array
   *   Array of menu items with url.
   */
  private function buildMenu(array $menu_items) {
    foreach ($menu_items as $key => $item) {
      $menu_items[$key]['url'] = str_replace('/drupal', '', $item['url']->toString());
      if (!empty($menu_items[$key]['below'])) {
        // There's items below. Call funtions recursively to build next level.
        $menu_items[$key]['below'] = $this->buildMenu($menu_items[$key]['below']);
      }
    }
    return $menu_items;
  }

}
